﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace fejvagyiras
{
    /*Ez a megoldás nem tárolja a beérkező adatokat, ugyanakkor az adatok alapján egy statisztikát tárol.
    A eredeti adatsorozat memóriafoglalása 1 byte/adat, az alábbi osztály 5 byte / sorozat. Akkor jelent memóriacsökkenést, ha a sorozatok átlaghossza > 5.
    Mivel a feladat kérdezi az FF számát, a 2 hosszúságú mintát értelmezni kell.
    A legrosszabb sorozat FF és II váltakozásával 2,5-szeresére növeli a memóriaigényt.
    Az elfogadhatóság határán van a megoldás, mert bár az adatokat nem tárolja, a memórián mégsem spórol.
    */
    class Sorozat
    {
        public char Tipus { get; set; }
        public int Hossz { get; set; }

        public Sorozat(char tipus, int hossz)
        {
            Tipus = tipus;
            Hossz = hossz;
        }

        public int dbF()
        {
            int db = 0;
            if (Tipus == 'F')
            {
                db = Hossz;
            }
            else if (Tipus == '1')
            {
                db = (Hossz - 1) / 2 + 1;
            }
            else if (Tipus == '2')
            {
                db = Hossz / 2;
            }
            return db;
        }
    }
    class Program
    {
        static Random rnd = new Random();

        static List<Sorozat> minta = new List<Sorozat>();

        static void Main(string[] args)
        {
            //Pénzfeldobás szimulálása
            Feladat01();
            //Egy dobás kiértékelése
            Feladat02();
            //fájl feldolgozása (feltételezem, hogy a kisérlet legalább 3 dobásból áll)
            Input();
            //adatok száma
            int db = Feladat03();
            //Fejek aránya
            Feladat04(db);
            //FF száma
            Feladat05();
            //leghosszabb fej sorozat hossza, helye
            Feladat06();
            //FFFF és FFFI száma 1000 sorozatban, fájlbaírás
            Feladat07();
            Console.ReadLine();
        }


        private static void Input()
        {
            char uj, elozo;
            StreamReader sr = new StreamReader("kiserlet.txt");
            elozo = sr.ReadLine()[0];
            char tipus = elozo == 'F' ? '1' : '2';
            Sorozat ujminta = new Sorozat(tipus, 1);

            while (!sr.EndOfStream)
            {
                uj = sr.ReadLine()[0];
                if (ujminta.Tipus == '1' || ujminta.Tipus == '2')
                {   //az aktuális sorozat váltakozó vagy egyelemű 1: F[...IF] v F[...FI] 2: I[...IF] v I[...FI] típusú
                    if (elozo != uj)
                    {   //ismét vált
                        ujminta.Hossz++;
                    }
                    else
                    {   //egyformák lesznek ezért az elozo már az új sorozat első eleme
                        ujminta.Hossz--;
                        if (ujminta.Hossz > 0)
                        {   //eddig váltakozott - ha Hossz ==1 akkor pl: ...IIFII... vagy ...FFIFF...
                            minta.Add(ujminta);
                            ujminta = new Sorozat(uj, 2);
                        }
                        else
                        {   //váltakozó Hossz ==0, ezért típus módosítása 1-->F, 2-->I
                            ujminta.Tipus = uj;
                            ujminta.Hossz = 2;
                        }
                    }
                }
                else
                {   //az aktuális sorozat homogén pl F: FF... I: II...
                    if (elozo == uj)
                    { //hosszabb soorzat
                        ujminta.Hossz++;
                    }
                    else
                    { //eddig azonos volt, de most vált
                        minta.Add(ujminta);
                        ujminta = new Sorozat((uj == 'F' ? '1' : '2'), 1);
                    }
                }
                elozo = uj;
            }
            minta.Add(ujminta);
            sr.Close();
        }

        private static char Randobas()
        {
            char e = 'I';
            if (rnd.NextDouble() < 0.5)
            {
                e = 'F';
            }
            return e;
        }

        private static void Feladat01()
        {
            Console.WriteLine("1. feladat");
            Console.WriteLine("A pénzfeldobás eredménye: {0}", Randobas());
        }

        private static void Feladat02()
        {
            Console.WriteLine("2. feladat");
            Console.Write("Tippeljen (F/I): ");
            string tipp = Console.ReadLine();
            char dob = Randobas();
            Console.WriteLine("A tipp {0}, a dobás eredménye {1} volt.\n{2}", tipp[0], dob, tipp[0] == dob ? "Ön eltalálta." : "Ön nem találta el.");
        }

        private static int Feladat03()
        {
            Console.WriteLine("3. feladat");
            int db = 0;
            foreach (Sorozat s in minta)
            {
                db += s.Hossz;
            }
            Console.WriteLine("A kísérlet {0} dobásból állt.", db);
            //ajánlott tesztek:
            //Console.WriteLine("A sorozatok átlaghossza: {0}", db / minta.Count());
            //Console.WriteLine("Memóriafoglalás szorzó: {0:P2}", minta.Count()*5.0 / db);
            return db;
        }

        private static void Feladat04(int db)
        {
            Console.WriteLine("4. feladat");
            int dbF = 0;
            foreach (Sorozat s in minta)
            {
                dbF += s.dbF();
            }

            Console.WriteLine("A kísérlet során a fej relatív gyakorisága {0:P2} volt.", dbF * 1.0 / db);
        }

        private static void Feladat05()
        {
            Console.WriteLine("5. feladat");
            int dbDupla = 0;
            foreach (Sorozat s in minta)
            {
                if (s.Tipus == 'F' && s.Hossz == 2)
                {
                    dbDupla++;
                }
            }
            Console.WriteLine("A kísérlet során pontosan {0} alkalommal dobtak két fejet egymás után.", dbDupla);
        }

        private static void Feladat06()
        {
            Console.WriteLine("6. feladat");
            int max = 0;
            int maxi = 0;
            int eddig = 0;
            foreach (Sorozat s in minta)
            {
                if (s.Tipus == 'F' && s.Hossz > max)
                {
                    max = s.Hossz;
                    maxi = eddig + 1;
                }
                eddig += s.Hossz;
            }
            Console.WriteLine("A leghosszabb tiszta fej sorozat {0} tagból áll, kezdete a(z) {1}. dobás.", max, maxi);
        }

        private static void Feladat07()
        {
            Console.WriteLine("7. feladat");
            string teszt = "";
            int FFFF = 0;
            int FFFI = 0;
            for (int i = 0; i < 1000; i++)
            {
                string sorozat = "";
                for (int j = 0; j < 4; j++)
                {
                    sorozat += Randobas();
                }
                if (sorozat == "FFFF")
                {
                    FFFF++;
                }
                else if (sorozat == "FFFI")
                {
                    FFFI++;
                }
                teszt += sorozat + " ";
            }
            StreamWriter sw = new StreamWriter("dobasok.txt");
            sw.WriteLine("FFFF: {0}, FFFI: {1}", FFFF, FFFI);
            sw.WriteLine(teszt);
            sw.Close();
            Console.WriteLine("Fájlbaírás kész");
        }
    }
}
